--[[-----------------------------------------------------------------

    Karmakazi's Clock, FPS and memory -- Taken from Neil's kClock.

--]]-----------------------------------------------------------------

local SORT_BY_NAME = false
local ANCHORPOINT = "BOTTOMRIGHT"

    --========[ important functions ]========--

local function Addoncompare(a, b)
    return a.memory > b.memory
end

local function MemFormat(v)
    if (v > 1024) then
        return string.format("%.2f MiB", v / 1024)
    else
        return string.format("%.2f KiB", v)
    end
end

local function ColorGradient(perc, ...)
	if (perc > 1) then
		local r, g, b = select(select('#', ...) - 2, ...)
		return r, g, b
	elseif (perc < 0) then
		local r, g, b = ...
		return r, g, b
	end
	
	local num = select('#', ...) / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

local function TimeFormat(time)
	local t = format("%.1ds",floor(mod(time,60)))
	if (time > 60) then
		time = floor(time / 60)
		t = format("%.1dm ",mod(time,60))..t
		if (time > 60) then
			time = floor(time / 60)
			t = format("%.1dh ",mod(time,24))..t
			if (time > 24) then
				time = floor(time / 24)
				t = format("%dd ",time)..t
			end
		end
	end
	return t
end

local function ColorizeLatency(v)
    if (v < 100) then
        return {r = 0, g = 1, b = 0}
    elseif (v < 300) then
        return {r = 1, g = 1, b = 0}
    else
        return {r = 1, g = 0, b = 0}
    end
end

local function ColorizeFramerate(v)
    if (v < 10) then
        return {r = 1, g = 0, b = 0}
    elseif (v < 30) then
        return {r = 1, g = 1, b = 0}
    else
        return {r = 0, g = 1, b = 0}
    end
end

    --========[ frames ]========--

local color = RAID_CLASS_COLORS[select(2, UnitClass("player"))]

local kClock = CreateFrame("Button", "kClock", UIParent)
kClock:SetPoint(ANCHORPOINT, UIParent)
kClock:SetHeight(20)
kClock:SetWidth(200)
kClock:SetMovable(true)
kClock:SetUserPlaced(true)

local text = kClock:CreateFontString(nil, "ARTWORK")
text:SetPoint("LEFT", kClock)
text:SetFont("Interface\\AddOns\\kSettings\\media\\ROADWAY.ttf", 18)
text:SetShadowOffset(1, -1)
text:SetTextColor(color.r, color.g, color.b)

local seconds = kClock:CreateFontString(nil, "ARTWORK")
seconds:SetPoint("BOTTOMLEFT", text, "BOTTOMRIGHT", 2, 1.8)
seconds:SetFont("Interface\\AddOns\\kSettings\\media\\ROADWAY.ttf", 14)
seconds:SetShadowOffset(1, -1)
seconds:SetTextColor(1, 1, 1)

    --========[ update ]========--

local lastUpdate = 0
local updateDelay = 1
kClock:SetScript("OnUpdate", function(self, elapsed)
    lastUpdate = lastUpdate + elapsed
    if (lastUpdate > updateDelay) then
        lastUpdate = 0
        local time, sec = date("%H|c00ffffff\46|r%M|c00ffffff|r"), date("%S")   -- local mem = " .. |cffffffff"..floor(collectgarbage("count") / 1024).."|rmb"     .......     local ping = "|c00ffffff"..select(3, GetNetStats()).."|r"
		fps = GetFramerate()
		fps = "|c00ffffff"..floor(fps).."|r fps  "
		lag = select(3, GetNetStats())
		lag = "|c00ffffff"..lag.."|r ms  "
        text:SetText(fps..lag..time)
        --seconds:SetText(sec)
    end
end)

    --========[ make the frame movable ]========--
  
kClock:SetScript("OnMouseDown", function()
    if (IsAltKeyDown()) then
        kClock:ClearAllPoints()
        kClock:StartMoving()
    end
end)
    kClock:SetScript("OnMouseUp", function()
        kClock:StopMovingOrSizing()
end)

    --========[ tooltip ]========--

kClock:SetScript("OnEnter", function()
    GameTooltip:SetOwner(kClock, "ANCHOR_TOPLEFT", 2, 5)
    collectgarbage()
    local memory, i, addons, total, entry, total
    local latencycolor = ColorizeLatency(select(3, GetNetStats()))
    local fpscolor = ColorizeFramerate(GetFramerate())
        
    GameTooltip:AddLine(date("%A, %d %B, %Y"), 1, 1, 1)
    GameTooltip:AddLine(" ") 
    GameTooltip:AddDoubleLine("Framerate:", format("%.1f fps", GetFramerate()), color.r, color.g, color.b, fpscolor.r, fpscolor.g, fpscolor.b)
    GameTooltip:AddDoubleLine("Latency:", format("%d ms", select(3, GetNetStats())), color.r, color.g, color.b, latencycolor.r, latencycolor.g, latencycolor.b)
    GameTooltip:AddLine(" ") 
    GameTooltip:AddDoubleLine("System Uptime:", TimeFormat(GetTime()), color.r, color.g, color.b, 1, 1, 1)
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine(". . . . . . . . . . .", ". . . . . . . . . . .", 1, 1, 1, 1, 1, 1)
    GameTooltip:AddLine(" ")
    GameTooltip:AddLine("AddOns:", color.r, color.g, color.b)
    GameTooltip:AddLine(" ")

    addons = {}
    total = 0
    UpdateAddOnMemoryUsage()
    for i = 1, GetNumAddOns(), 1 do
        if GetAddOnMemoryUsage(i) > 0 then
            memory = GetAddOnMemoryUsage(i)
            entry = {name = GetAddOnInfo(i), memory = memory}
            table.insert(addons, entry)
            total = total + memory
        end
    end
    
    if (SORT_BY_NAME == false) then 
        table.sort(addons, Addoncompare)
    end

   for _,entry in pairs(addons) do
		local cr, cg, cb = ColorGradient((entry.memory / 800), 0, 1, 0, 1, 1, 0, 1, 0, 0)
		GameTooltip:AddDoubleLine(entry.name, MemFormat(entry.memory), 1, 1, 1, cr, cg, cb)
	end
    local cr, cg, cb = ColorGradient((entry.memory / 800), 0, 1, 0, 1, 1, 0, 1, 0, 0) 
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine(". . . . . . . . . . .", ". . . . . . . . . . .", 1, 1, 1, 1, 1, 1)
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine("Total", MemFormat(total), color.r, color.g, color.b, cr, cg, cb)
    GameTooltip:AddDoubleLine("..with Blizzard", MemFormat(collectgarbage("count")), color.r, color.g, color.b, cr, cg, cb)
    GameTooltip:Show()
end)

kClock:SetScript("OnLeave", function() 
    GameTooltip:Hide() 
end)

    --========[ mem cleanup ]========--

kClock:SetScript("OnClick", function()
    if (not IsAltKeyDown()) then
        UpdateAddOnMemoryUsage()
        local memBefore = gcinfo()
        collectgarbage()
        UpdateAddOnMemoryUsage()
        local memAfter = gcinfo()
        DEFAULT_CHAT_FRAME:AddMessage("Memory cleaned: |cff00FF00"..MemFormat(memBefore - memAfter))
    end
end)